@@ -1,12 +1,8 @@
 from enum import Enum
 
-from dagster.check import not_implemented
-
 
 class TaskStatus(Enum):
-    """
-    Enum for the status of a task.
-    """
+    """Enum for the status of a task."""
 
     RUNNING = "RUNNING"
     SUCCESS = "SUCCESS"
