@@ -1,16 +1,14 @@
 import os
 import sys
-from typing import List
+from collections.abc import Sequence
 
-from dagster.serdes.ipc import interrupt_ipc_subprocess_pid, open_ipc_subprocess
+from dagster_shared.ipc import interrupt_ipc_subprocess_pid, open_ipc_subprocess
 
-from . import TaskStatus
+from dagster_cloud.execution.utils import TaskStatus
 
 
-def launch_process(args: List[str]) -> int:
-    """
-    Launch a process and return the PID
-    """
+def launch_process(args: Sequence[str]) -> int:
+    """Launch a process and return the PID."""
     p = open_ipc_subprocess(args)
     pid = p.pid
     return pid
