@@ -43,20 +43,34 @@ ADD_RUN_MUTATION = (
 """
 )
 
-HANDLE_RUN_EVENT_MUTATION = """
-    mutation handleRunEventMutation($runId: String!, $serializedEvent: String!) {
+ADD_HISTORICAL_RUN_MUTATION = (
+    ERROR_FRAGMENT
+    + """
+    mutation addHistoricalRunMutation($serializedPipelineRun: String!, $runCreationTime: Float!) {
         runs {
-            handleRunEvent(runId: $runId, serializedEvent: $serializedEvent) {
+            addHistoricalRun(serializedPipelineRun: $serializedPipelineRun, runCreationTime: $runCreationTime) {
                 ok
+                error {
+                    ...errorFragment
+                }
             }
         }
     }
+    """
+)
+
+GET_RUNS_QUERY = """
+    query getRunsQuery($filters: RunsFilter, $cursor: String, $limit: Int, $bucketBy: RunBucket, $ascending: Boolean) {
+        runs {
+            getRuns(filters: $filters, cursor: $cursor, limit: $limit, bucketBy: $bucketBy, ascending: $ascending)
+        }
+    }
 """
 
-GET_RUNS_QUERY = """
-    query getRunsQuery($filters: RunsFilter, $cursor: String, $limit: Int, $bucketBy: RunBucket) {
+GET_RUN_IDS_QUERY = """
+    query getRunsIdsQuery($filters: RunsFilter, $cursor: String, $limit: Int) {
         runs {
-            getRuns(filters: $filters, cursor: $cursor, limit: $limit, bucketBy: $bucketBy)
+            getRunIds(filters: $filters, cursor: $cursor, limit: $limit)
         }
     }
 """
@@ -119,24 +133,12 @@ GET_RUN_GROUP_QUERY = """
 """
 
 
-GET_RUN_GROUPS_QUERY = """
-    query getRunGroupsQuery($filters: RunsFilter, $cursor: String, $limit: Int) {
-        runs {
-            getRunGroups(filters: $filters, cursor: $cursor, limit: $limit) {
-                rootRunId
-                serializedRuns
-                count
-            }
-        }
-    }
-"""
-
 GET_RUN_RECORDS_QUERY = (
     RUN_ROW_FRAGMENT
     + """
-    query getRunRecordsQuery($filters: RunsFilter, $limit: Int, $orderBy: String, $ascending: Boolean, $bucketBy: RunBucket) {
+    query getRunRecordsQuery($filters: RunsFilter, $limit: Int, $orderBy: String, $ascending: Boolean, $cursor: String, $bucketBy: RunBucket) {
         runs {
-            getRunRecords(filters: $filters, limit: $limit, orderBy: $orderBy, ascending: $ascending, bucketBy: $bucketBy) {
+            getRunRecords(filters: $filters, limit: $limit, orderBy: $orderBy, ascending: $ascending, cursor: $cursor, bucketBy: $bucketBy) {
                 ...runRecordFragment
             }
         }
@@ -145,9 +147,9 @@ GET_RUN_RECORDS_QUERY = (
 )
 
 GET_RUN_TAGS_QUERY = """
-    query getRunTagsQuery {
+    query getRunTagsQuery($jsonTagKeys: JSONString!, $valuePrefix: String, $limit: Int) {
         runs {
-            getRunTags {
+            getRunTags(jsonTagKeys: $jsonTagKeys, valuePrefix: $valuePrefix, limit: $limit) {
                 key
                 values
             }
@@ -155,6 +157,15 @@ GET_RUN_TAGS_QUERY = """
     }
 """
 
+GET_RUN_TAG_KEYS_QUERY = """
+    query getRunTagKeysQuery {
+        runs {
+            getRunTagKeys
+        }
+    }
+"""
+
+
 ADD_RUN_TAGS_MUTATION = """
     mutation addRunTagsMutation($runId: String!, $jsonNewTags: JSONString!) {
         runs {
@@ -219,10 +230,20 @@ GET_DAEMON_HEARTBEATS_QUERY = """
     }
 """
 
-GET_BACKFILLS_QUERY = """
-    query getBackfillsQuery($status: String, $cursor: String, $limit: Int) {
+ADD_RUN_TELEMETRY_MUTATION = """
+    mutation addRunTelemetry($serializedTelemetry: String!, $serializedTags: String!) {
         runs {
-            getBackfills(status: $status, cursor: $cursor, limit: $limit)
+            addRunTelemetry(serializedTelemetry: $serializedTelemetry, serializedTags: $serializedTags) {
+                ok
+            }
+        }
+    }
+"""
+
+GET_BACKFILLS_QUERY = """
+    query getBackfillsQuery($status: String, $cursor: String, $limit: Int, $filters: BulkActionsFilter) {
+        runs {
+            getBackfills(status: $status, cursor: $cursor, limit: $limit, filters: $filters)
         }
     }
 """
@@ -254,3 +275,21 @@ UPDATE_BACKFILL_MUTATION = """
         }
     }
 """
+
+GET_RUN_PARTITION_DATA_QUERY = """
+    query getRunPartitionData($runsFilter: RunsFilter!) {
+        runs {
+            getRunPartitionData(runsFilter: $runsFilter)
+        }
+    }
+"""
+
+MUTATE_JOB_ORIGIN = """
+    mutation mutateJobOrigin($runId: String!, $serializedJobOrigin: String!) {
+        runs {
+            mutateJobOrigin(runId: $runId, serializedJobOrigin: $serializedJobOrigin) {
+                ok
+            }
+        }
+    }
+"""
