@@ -1,102 +1,140 @@
 import json
-from typing import Any, Callable, Dict, Iterable, List, Optional, Set, Tuple, Union
+from collections.abc import Iterable, Mapping, Sequence
+from datetime import datetime
+from typing import Any, Callable, Optional, Union
 
-from dagster import check
-from dagster.core.errors import (
+import dagster._check as check
+from dagster._core.errors import (
     DagsterInvariantViolationError,
     DagsterRunAlreadyExists,
     DagsterRunNotFoundError,
     DagsterSnapshotDoesNotExist,
 )
-from dagster.core.events import DagsterEvent
-from dagster.core.execution.backfill import BulkActionStatus, PartitionBackfill
-from dagster.core.snap import (
-    ExecutionPlanSnapshot,
-    PipelineSnapshot,
-    create_execution_plan_snapshot_id,
-    create_pipeline_snapshot_id,
-)
-from dagster.core.storage.pipeline_run import (
+from dagster._core.events import DagsterEvent
+from dagster._core.execution.backfill import BulkActionsFilter, BulkActionStatus, PartitionBackfill
+from dagster._core.execution.telemetry import RunTelemetryData
+from dagster._core.remote_origin import RemoteJobOrigin
+from dagster._core.snap import ExecutionPlanSnapshot, JobSnap, create_execution_plan_snapshot_id
+from dagster._core.storage.dagster_run import (
+    DagsterRun,
     JobBucket,
-    PipelineRun,
-    PipelineRunsFilter,
+    RunPartitionData,
     RunRecord,
+    RunsFilter,
     TagBucket,
 )
-from dagster.core.storage.runs.base import RunStorage
-from dagster.daemon.types import DaemonHeartbeat
-from dagster.serdes import (
+from dagster._core.storage.runs.base import RunStorage
+from dagster._daemon.types import DaemonHeartbeat
+from dagster._serdes import (
     ConfigurableClass,
     ConfigurableClassData,
-    deserialize_as,
-    deserialize_json_to_dagster_namedtuple,
-    serialize_dagster_namedtuple,
+    deserialize_value,
+    serialize_value,
 )
-from dagster.utils import utc_datetime_from_timestamp
+from dagster._time import datetime_from_timestamp
+from dagster._utils.merger import merge_dicts
+from dagster_cloud_cli.core.errors import DagsterCloudAgentServerError
+from typing_extensions import Self
 
-from ..errors import GraphQLStorageError
-from .queries import (
+from dagster_cloud.storage.runs.queries import (
     ADD_BACKFILL_MUTATION,
     ADD_DAEMON_HEARTBEAT_MUTATION,
     ADD_EXECUTION_PLAN_SNAPSHOT_MUTATION,
+    ADD_HISTORICAL_RUN_MUTATION,
     ADD_PIPELINE_SNAPSHOT_MUTATION,
     ADD_RUN_MUTATION,
     ADD_RUN_TAGS_MUTATION,
-    GET_BACKFILLS_QUERY,
+    ADD_RUN_TELEMETRY_MUTATION,
     GET_BACKFILL_QUERY,
+    GET_BACKFILLS_QUERY,
     GET_DAEMON_HEARTBEATS_QUERY,
     GET_EXECUTION_PLAN_SNAPSHOT_QUERY,
     GET_PIPELINE_SNAPSHOT_QUERY,
+    GET_RUN_BY_ID_QUERY,
+    GET_RUN_GROUP_QUERY,
+    GET_RUN_IDS_QUERY,
+    GET_RUN_PARTITION_DATA_QUERY,
+    GET_RUN_RECORDS_QUERY,
+    GET_RUN_TAG_KEYS_QUERY,
+    GET_RUN_TAGS_QUERY,
     GET_RUNS_COUNT_QUERY,
     GET_RUNS_QUERY,
-    GET_RUN_BY_ID_QUERY,
-    GET_RUN_GROUPS_QUERY,
-    GET_RUN_GROUP_QUERY,
-    GET_RUN_RECORDS_QUERY,
-    GET_RUN_TAGS_QUERY,
-    HANDLE_RUN_EVENT_MUTATION,
     HAS_EXECUTION_PLAN_SNAPSHOT_QUERY,
     HAS_PIPELINE_SNAPSHOT_QUERY,
     HAS_RUN_QUERY,
+    MUTATE_JOB_ORIGIN,
     UPDATE_BACKFILL_MUTATION,
 )
 
 
-def _get_filters_input(filters) -> Optional[Dict[str, Any]]:
-    filters = check.opt_inst_param(filters, "filters", PipelineRunsFilter)
+def _get_filters_input(filters: Optional[RunsFilter]) -> Optional[dict[str, Any]]:
+    filters = check.opt_inst_param(filters, "filters", RunsFilter)
 
     if filters is None:
         return None
+
+    check.invariant(filters.exclude_subruns is None, "RunsFilter.exclude_subruns is not supported")
     return {
         "runIds": filters.run_ids,
-        "pipelineName": filters.pipeline_name,
-        "mode": filters.mode,
+        "pipelineName": filters.job_name,
         "statuses": [status.value for status in filters.statuses],
-        "tags": [
-            {"key": tag_key, "value": tag_value} for tag_key, tag_value in filters.tags.items()
-        ]
-        if filters.tags
-        else [],
+        "tags": (
+            [
+                merge_dicts(
+                    {"key": tag_key},
+                    ({"value": tag_value} if isinstance(tag_value, str) else {"values": tag_value}),
+                )
+                for tag_key, tag_value in filters.tags.items()
+            ]
+            if filters.tags
+            else []
+        ),
         "snapshotId": filters.snapshot_id,
         "updatedAfter": filters.updated_after.timestamp() if filters.updated_after else None,
+        "updatedBefore": filters.updated_before.timestamp() if filters.updated_before else None,
+        "createdAfter": filters.created_after.timestamp() if filters.created_after else None,
         "createdBefore": filters.created_before.timestamp() if filters.created_before else None,
     }
 
 
-def _run_record_from_graphql(graphene_run_record: Dict) -> RunRecord:
+def _get_bulk_actions_filters_input(
+    filters: Optional[BulkActionsFilter],
+) -> Optional[dict[str, Any]]:
+    filters = check.opt_inst_param(filters, "filters", BulkActionsFilter)
+    unsupported_filters = []
+    if filters and filters.job_name:
+        unsupported_filters.append("job_name")
+    if filters and filters.backfill_ids:
+        unsupported_filters.append("backfill_ids")
+    if filters and filters.tags:
+        unsupported_filters.append("tags")
+
+    check.invariant(
+        len(unsupported_filters) == 0,
+        f"Used the following unsupported filters: {', '.join(unsupported_filters)}.",
+    )
+
+    if filters is None:
+        return None
+    return {
+        "statuses": [status.value for status in filters.statuses] if filters.statuses else None,
+        "createdAfter": filters.created_after.timestamp() if filters.created_after else None,
+        "createdBefore": filters.created_before.timestamp() if filters.created_before else None,
+    }
+
+
+def _run_record_from_graphql(graphene_run_record: dict) -> RunRecord:
     check.dict_param(graphene_run_record, "graphene_run_record")
     return RunRecord(
         storage_id=check.int_elem(graphene_run_record, "storageId"),
-        pipeline_run=check.inst(
-            deserialize_json_to_dagster_namedtuple(
-                check.str_elem(graphene_run_record, "serializedPipelineRun")
-            ),
-            PipelineRun,
+        dagster_run=deserialize_value(
+            check.str_elem(graphene_run_record, "serializedPipelineRun"),
+            DagsterRun,
         ),
-        create_timestamp=utc_datetime_from_timestamp(
+        create_timestamp=datetime_from_timestamp(
             check.float_elem(graphene_run_record, "createTimestamp")
         ),
-        update_timestamp=utc_datetime_from_timestamp(
+        update_timestamp=datetime_from_timestamp(
             check.float_elem(graphene_run_record, "updateTimestamp")
         ),
         start_time=graphene_run_record.get("startTime"),
@@ -104,7 +142,7 @@ def _run_record_from_graphql(graphene_run_record: Dict) -> RunRecord:
     )
 
 
-def _get_bucket_input(bucket_by: Optional[Union[JobBucket, TagBucket]]) -> Optional[Dict[str, Any]]:
+def _get_bucket_input(bucket_by: Optional[Union[JobBucket, TagBucket]]) -> Optional[dict[str, Any]]:
     if not bucket_by:
         return None
 
@@ -120,9 +158,12 @@ def _get_bucket_input(bucket_by: Optional[Union[JobBucket, TagBucket]]) -> Optio
 
 
 class GraphQLRunStorage(RunStorage, ConfigurableClass):
-    def __init__(self, inst_data: ConfigurableClassData = None, override_graphql_client=None):
+    def __init__(
+        self, inst_data: Optional[ConfigurableClassData] = None, override_graphql_client=None
+    ):
         """Initialize this class directly only for test (using `override_graphql_client`).
-        Use the ConfigurableClass machinery to init from instance yaml."""
+        Use the ConfigurableClass machinery to init from instance yaml.
+        """
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
         self._override_graphql_client = override_graphql_client
 
@@ -134,29 +175,51 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
     def config_type(cls):
         return {}
 
-    @staticmethod
-    def from_config_value(inst_data: ConfigurableClassData, config_value):
-        return GraphQLRunStorage(inst_data=inst_data)
+    @classmethod
+    def from_config_value(cls, inst_data: ConfigurableClassData, config_value: Any) -> Self:
+        return cls(inst_data=inst_data)
 
     @property
     def _graphql_client(self):
         return (
             self._override_graphql_client
             if self._override_graphql_client
-            else self._instance.graphql_client
+            else self._instance.graphql_client  # pyright: ignore[reportAttributeAccessIssue]
         )
 
-    def _execute_query(self, query, variables=None):
-        res = self._graphql_client.execute(query, variable_values=variables)
-        if "errors" in res:
-            raise GraphQLStorageError(res)
-        return res
+    def _execute_query(self, query, variables=None, idempotent_mutation=False):
+        return self._graphql_client.execute(
+            query, variable_values=variables, idempotent_mutation=idempotent_mutation
+        )
 
-    def add_run(self, pipeline_run: PipelineRun):
-        check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
+    def add_historical_run(
+        self, dagster_run: DagsterRun, run_creation_time: datetime
+    ) -> DagsterRun:
+        check.inst_param(dagster_run, "dagster_run", DagsterRun)
+        res = self._execute_query(
+            ADD_HISTORICAL_RUN_MUTATION,
+            variables={
+                "serializedPipelineRun": serialize_value(dagster_run),
+                "runCreationTime": run_creation_time.timestamp(),
+            },
+        )
+        result = res["data"]["runs"]["addHistoricalRun"]
+        error = result.get("error")
+        # Special-case some errors to match the RunStorage API
+        if error:
+            if error["className"] == "DagsterRunAlreadyExists":
+                raise DagsterRunAlreadyExists(error["message"])
+            if error["className"] == "DagsterSnapshotDoesNotExist":
+                raise DagsterSnapshotDoesNotExist(error["message"])
+            else:
+                raise DagsterCloudAgentServerError(res)
+        return dagster_run
+
+    def add_run(self, dagster_run: DagsterRun):
+        check.inst_param(dagster_run, "dagster_run", DagsterRun)
         res = self._execute_query(
             ADD_RUN_MUTATION,
-            variables={"serializedPipelineRun": serialize_dagster_namedtuple(pipeline_run)},
+            variables={"serializedPipelineRun": serialize_value(dagster_run)},
         )
 
         result = res["data"]["runs"]["addRun"]
@@ -169,27 +232,27 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
             if error["className"] == "DagsterSnapshotDoesNotExist":
                 raise DagsterSnapshotDoesNotExist(error["message"])
             else:
-                raise GraphQLStorageError(res)
+                raise DagsterCloudAgentServerError(res)
 
-        return pipeline_run
+        return dagster_run
 
-    def handle_run_event(self, run_id: str, event: DagsterEvent):
-        check.inst_param(event, "event", DagsterEvent)
-        self._execute_query(
-            HANDLE_RUN_EVENT_MUTATION,
-            variables={
-                "runId": check.str_param(run_id, "run_id"),
-                "serializedEvent": serialize_dagster_namedtuple(event),
-            },
-        )
+    def handle_run_event(
+        self, run_id: str, event: DagsterEvent, update_timestamp: Optional[datetime] = None
+    ):
+        raise NotImplementedError("Should never be called by an agent client")
 
-    def get_runs(
+    @property
+    def supports_bucket_queries(self) -> bool:
+        return False
+
+    def get_runs(  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
         self,
-        filters: PipelineRunsFilter = None,
-        cursor: str = None,
-        limit: int = None,
+        filters: Optional[RunsFilter] = None,
+        cursor: Optional[str] = None,
+        limit: Optional[int] = None,
         bucket_by: Optional[Union[JobBucket, TagBucket]] = None,
-    ) -> Iterable[PipelineRun]:
+        ascending: bool = False,
+    ) -> Iterable[DagsterRun]:
         res = self._execute_query(
             GET_RUNS_QUERY,
             variables={
@@ -197,11 +260,28 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
                 "cursor": check.opt_str_param(cursor, "cursor"),
                 "limit": check.opt_int_param(limit, "limit"),
                 "bucketBy": _get_bucket_input(bucket_by),
+                "ascending": check.bool_param(ascending, "ascending"),
             },
         )
-        return [deserialize_as(run, PipelineRun) for run in res["data"]["runs"]["getRuns"]]
+        return [deserialize_value(run, DagsterRun) for run in res["data"]["runs"]["getRuns"]]
 
-    def get_runs_count(self, filters: PipelineRunsFilter = None) -> int:
+    def get_run_ids(
+        self,
+        filters: Optional[RunsFilter] = None,
+        cursor: Optional[str] = None,
+        limit: Optional[int] = None,
+    ) -> Sequence[str]:
+        res = self._execute_query(
+            GET_RUN_IDS_QUERY,
+            variables={
+                "filters": _get_filters_input(filters),
+                "cursor": check.opt_str_param(cursor, "cursor"),
+                "limit": check.opt_int_param(limit, "limit"),
+            },
+        )
+        return res["data"]["runs"]["getRunIds"]
+
+    def get_runs_count(self, filters: Optional[RunsFilter] = None) -> int:
         res = self._execute_query(
             GET_RUNS_COUNT_QUERY,
             variables={
@@ -210,7 +290,7 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
         )
         return res["data"]["runs"]["getRunsCount"]
 
-    def get_run_group(self, run_id: str) -> Optional[Tuple[str, Iterable[PipelineRun]]]:
+    def get_run_group(self, run_id: str) -> Optional[tuple[str, Iterable[DagsterRun]]]:  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
         res = self._execute_query(
             GET_RUN_GROUP_QUERY, variables={"runId": check.str_param(run_id, "run_id")}
         )
@@ -218,37 +298,17 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
         if run_group_or_error["__typename"] == "SerializedRunGroup":
             return (
                 run_group_or_error["rootRunId"],
-                [deserialize_as(run, PipelineRun) for run in run_group_or_error["serializedRuns"]],
+                [
+                    deserialize_value(run, DagsterRun)
+                    for run in run_group_or_error["serializedRuns"]
+                ],
             )
         elif run_group_or_error["__typename"] == "RunNotFoundError":
             raise DagsterRunNotFoundError(invalid_run_id=run_group_or_error["runId"])
         else:
-            raise DagsterInvariantViolationError(
-                f"Unexpected getRunGroupOrError response {str(res)}"
-            )
+            raise DagsterInvariantViolationError(f"Unexpected getRunGroupOrError response {res}")
 
-    def get_run_groups(
-        self, filters: PipelineRunsFilter = None, cursor: str = None, limit: int = None
-    ) -> Dict[str, Dict[str, Union[Iterable[PipelineRun], int]]]:
-        res = self._execute_query(
-            GET_RUN_GROUPS_QUERY,
-            variables={
-                "filters": _get_filters_input(filters),
-                "cursor": check.opt_str_param(cursor, "cursor"),
-                "limit": check.opt_int_param(limit, "limit"),
-            },
-        )
-        raw_run_groups = res["data"]["runs"]["getRunGroups"]
-
-        run_groups = {}
-        for run_group in raw_run_groups:
-            run_groups[run_group["rootRunId"]] = {
-                "runs": [deserialize_as(run, PipelineRun) for run in run_group["serializedRuns"]],
-                "count": run_group["count"],
-            }
-        return run_groups
-
-    def get_run_by_id(self, run_id: str) -> Optional[PipelineRun]:
+    def get_run_by_id(self, run_id: str) -> Optional[DagsterRun]:
         res = self._execute_query(
             GET_RUN_BY_ID_QUERY, variables={"runId": check.str_param(run_id, "run_id")}
         )
@@ -256,17 +316,17 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
         if run is None:
             return None
 
-        return deserialize_as(run, PipelineRun)
+        return deserialize_value(run, DagsterRun)
 
     def get_run_records(
         self,
-        filters: PipelineRunsFilter = None,
-        limit: int = None,
-        order_by: str = None,
+        filters: Optional[RunsFilter] = None,
+        limit: Optional[int] = None,
+        order_by: Optional[str] = None,
         ascending: bool = False,
-        cursor: str = None,
+        cursor: Optional[str] = None,
         bucket_by: Optional[Union[JobBucket, TagBucket]] = None,
-    ) -> List[RunRecord]:
+    ) -> list[RunRecord]:
         res = self._execute_query(
             GET_RUN_RECORDS_QUERY,
             variables={
@@ -280,22 +340,39 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
         )
         return [_run_record_from_graphql(record) for record in res["data"]["runs"]["getRunRecords"]]
 
-    def get_run_tags(self) -> List[Tuple[str, Set[str]]]:
-        res = self._execute_query(GET_RUN_TAGS_QUERY)
+    def get_run_tags(
+        self,
+        tag_keys: Sequence[str],
+        value_prefix: Optional[str] = None,
+        limit: Optional[int] = None,
+    ) -> Sequence[tuple[str, set[str]]]:
+        res = self._execute_query(
+            GET_RUN_TAGS_QUERY,
+            variables={
+                "jsonTagKeys": (json.dumps(check.list_param(tag_keys, "tag_keys", of_type=str))),
+                "valuePrefix": check.opt_str_param(value_prefix, "value_prefix"),
+                "limit": check.opt_int_param(limit, "limit"),
+            },
+        )
         return [
             (run_tag["key"], set(run_tag["values"]))
             for run_tag in res["data"]["runs"]["getRunTags"]
         ]
 
-    def add_run_tags(self, run_id: str, new_tags: Dict[str, str]):
+    def get_run_tag_keys(self) -> Sequence[str]:
+        res = self._execute_query(GET_RUN_TAG_KEYS_QUERY)
+        return [run_tag_key for run_tag_key in res["data"]["runs"]["getRunTagKeys"]]
+
+    def add_run_tags(self, run_id: str, new_tags: Mapping[str, str]):
         self._execute_query(
             ADD_RUN_TAGS_MUTATION,
             variables={
                 "runId": check.str_param(run_id, "run_id"),
                 "jsonNewTags": json.dumps(
-                    check.dict_param(new_tags, "new_tags", key_type=str, value_type=str)
+                    check.mapping_param(new_tags, "new_tags", key_type=str, value_type=str)
                 ),
             },
+            idempotent_mutation=True,
         )
 
     def has_run(self, run_id: str) -> bool:
@@ -304,7 +381,7 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
         )
         return res["data"]["runs"]["hasRun"]
 
-    def has_pipeline_snapshot(self, pipeline_snapshot_id: str) -> bool:
+    def has_job_snapshot(self, pipeline_snapshot_id: str) -> bool:  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
         res = self._execute_query(
             HAS_PIPELINE_SNAPSHOT_QUERY,
             variables={
@@ -313,31 +390,28 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
         )
         return res["data"]["runs"]["hasPipelineSnapshot"]
 
-    def add_pipeline_snapshot(
-        self, pipeline_snapshot: PipelineSnapshot, snapshot_id: Optional[str] = None
+    def add_job_snapshot(  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
+        self, pipeline_snapshot: JobSnap, snapshot_id: Optional[str] = None
     ) -> str:
         self._execute_query(
             ADD_PIPELINE_SNAPSHOT_MUTATION,
             variables={
-                "serializedPipelineSnapshot": serialize_dagster_namedtuple(
-                    check.inst_param(pipeline_snapshot, "pipeline_snapshot", PipelineSnapshot)
+                "serializedPipelineSnapshot": serialize_value(
+                    check.inst_param(pipeline_snapshot, "pipeline_snapshot", JobSnap)
                 ),
                 "snapshotId": snapshot_id,
             },
         )
-        return snapshot_id if snapshot_id else create_pipeline_snapshot_id(pipeline_snapshot)
+        return snapshot_id if snapshot_id else pipeline_snapshot.snapshot_id
 
-    def get_pipeline_snapshot(self, pipeline_snapshot_id: str) -> PipelineSnapshot:
+    def get_job_snapshot(self, pipeline_snapshot_id: str) -> JobSnap:  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
         res = self._execute_query(
             GET_PIPELINE_SNAPSHOT_QUERY,
             variables={
                 "pipelineSnapshotId": check.str_param(pipeline_snapshot_id, "pipeline_snapshot_id")
             },
         )
-        return check.inst(
-            deserialize_json_to_dagster_namedtuple(res["data"]["runs"]["getPipelineSnapshot"]),
-            PipelineSnapshot,
-        )
+        return deserialize_value(res["data"]["runs"]["getPipelineSnapshot"], JobSnap)
 
     def has_execution_plan_snapshot(self, execution_plan_snapshot_id: str) -> bool:
         res = self._execute_query(
@@ -356,7 +430,7 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
         self._execute_query(
             ADD_EXECUTION_PLAN_SNAPSHOT_MUTATION,
             variables={
-                "serializedExecutionPlanSnapshot": serialize_dagster_namedtuple(
+                "serializedExecutionPlanSnapshot": serialize_value(
                     check.inst_param(
                         execution_plan_snapshot, "execution_plan_snapshot", ExecutionPlanSnapshot
                     )
@@ -379,30 +453,64 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
                 )
             },
         )
-        return check.inst(
-            deserialize_json_to_dagster_namedtuple(res["data"]["runs"]["getExecutionPlanSnapshot"]),
-            ExecutionPlanSnapshot,
+        return deserialize_value(
+            res["data"]["runs"]["getExecutionPlanSnapshot"], ExecutionPlanSnapshot
         )
 
+    def get_run_partition_data(self, runs_filter: RunsFilter) -> list[RunPartitionData]:
+        res = self._execute_query(
+            GET_RUN_PARTITION_DATA_QUERY,
+            variables={
+                "runsFilter": _get_filters_input(runs_filter),
+            },
+        )
+        return [
+            deserialize_value(result, RunPartitionData)
+            for result in res["data"]["runs"]["getRunPartitionData"]
+        ]
+
     def add_daemon_heartbeat(self, daemon_heartbeat: DaemonHeartbeat):
         self._execute_query(
             ADD_DAEMON_HEARTBEAT_MUTATION,
             variables={
-                "serializedDaemonHeartbeat": serialize_dagster_namedtuple(
+                "serializedDaemonHeartbeat": serialize_value(
                     check.inst_param(daemon_heartbeat, "daemon_heartbeat", DaemonHeartbeat)
                 )
             },
         )
 
+    def supports_run_telemetry(self) -> bool:
+        return True
+
+    def add_run_telemetry(
+        self,
+        run_telemetry: RunTelemetryData,
+        tags: Optional[dict[str, str]] = None,
+    ) -> None:
+        if tags is None:
+            tags = {}
+
+        self._execute_query(
+            ADD_RUN_TELEMETRY_MUTATION,
+            variables={
+                "serializedTelemetry": serialize_value(
+                    check.inst_param(run_telemetry, "run_telemetry", RunTelemetryData)
+                ),
+                "serializedTags": serialize_value(
+                    check.dict_param(tags, "tags", key_type=str, value_type=str)
+                ),
+            },
+        )
+
     def build_missing_indexes(
         self, print_fn: Callable = lambda _: None, force_rebuild_all: bool = False
     ):
         raise Exception("Not allowed to build indexes from user cloud")
 
-    def get_daemon_heartbeats(self) -> Dict[str, DaemonHeartbeat]:
+    def get_daemon_heartbeats(self) -> dict[str, DaemonHeartbeat]:
         res = self._execute_query(GET_DAEMON_HEARTBEATS_QUERY)
         return {
-            key: deserialize_as(heartbeat, DaemonHeartbeat)
+            key: deserialize_value(heartbeat, DaemonHeartbeat)
             for key, heartbeat in json.loads(res["data"]["runs"]["getDaemonHeartbeats"]).items()
         }
 
@@ -418,41 +526,63 @@ class GraphQLRunStorage(RunStorage, ConfigurableClass):
     def has_bulk_actions_table(self) -> bool:
         return False
 
-    def get_backfills(self, status: BulkActionStatus = None, cursor: str = None, limit: int = None):
-        """Get a list of partition backfills"""
+    def get_backfills(
+        self,
+        filters: Optional[BulkActionsFilter] = None,
+        cursor: Optional[str] = None,
+        limit: Optional[int] = None,
+        status: Optional[BulkActionStatus] = None,
+    ):
+        """Get a list of partition backfills."""
         res = self._execute_query(
             GET_BACKFILLS_QUERY,
             variables={
                 "status": status.value if status else None,
                 "cursor": check.opt_str_param(cursor, "cursor"),
                 "limit": check.opt_int_param(limit, "limit"),
+                "filters": _get_bulk_actions_filters_input(filters),
             },
         )
         return [
-            deserialize_as(backfill, PartitionBackfill)
+            deserialize_value(backfill, PartitionBackfill)
             for backfill in res["data"]["runs"]["getBackfills"]
         ]
 
+    def get_backfills_count(self, filters: Optional[BulkActionsFilter] = None) -> int:
+        raise NotImplementedError("get_backfills_count is not callable from user cloud.")
+
     def get_backfill(self, backfill_id: str) -> PartitionBackfill:
-        """Get a single partition backfill"""
+        """Get a single partition backfill."""
         res = self._execute_query(GET_BACKFILL_QUERY, variables={"backfillId": backfill_id})
         backfill = res["data"]["runs"]["getBackfill"]
-        return deserialize_as(backfill, PartitionBackfill)
+        return deserialize_value(backfill, PartitionBackfill)
 
     def add_backfill(self, partition_backfill: PartitionBackfill):
-        """Add partition backfill to run storage"""
+        """Add partition backfill to run storage."""
         self._execute_query(
             ADD_BACKFILL_MUTATION,
-            variables={
-                "serializedPartitionBackfill": serialize_dagster_namedtuple(partition_backfill)
-            },
+            variables={"serializedPartitionBackfill": serialize_value(partition_backfill)},
         )
 
     def update_backfill(self, partition_backfill: PartitionBackfill):
-        """Update a partition backfill in run storage"""
+        """Update a partition backfill in run storage."""
         self._execute_query(
             UPDATE_BACKFILL_MUTATION,
+            variables={"serializedPartitionBackfill": serialize_value(partition_backfill)},
+        )
+
+    def get_cursor_values(self, keys: set[str]):  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
+        return NotImplementedError("KVS is not supported from the user cloud")
+
+    def set_cursor_values(self, pairs: Mapping[str, str]):  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
+        return NotImplementedError("KVS is not supported from the user cloud")
+
+    # Migrating run history
+    def replace_job_origin(self, run: DagsterRun, job_origin: RemoteJobOrigin):
+        self._execute_query(
+            MUTATE_JOB_ORIGIN,
             variables={
-                "serializedPartitionBackfill": serialize_dagster_namedtuple(partition_backfill)
+                "runId": run.run_id,
+                "serializedJobOrigin": serialize_value(job_origin),
             },
         )
