@@ -1,40 +1,48 @@
-from typing import Iterable, List, Optional
+from collections.abc import Iterable, Sequence
+from typing import TYPE_CHECKING, Any, Optional
 
-from dagster import check
-from dagster.core.definitions.run_request import InstigatorType
-from dagster.core.scheduler.instigation import (
+import dagster._check as check
+from dagster._core.definitions.asset_key import EntityKey
+from dagster._core.definitions.declarative_automation.serialized_objects import (
+    AutomationConditionEvaluationWithRunIds,
+)
+from dagster._core.definitions.run_request import InstigatorType
+from dagster._core.scheduler.instigation import (
+    AutoMaterializeAssetEvaluationRecord,
     InstigatorState,
+    InstigatorStatus,
     InstigatorTick,
     TickData,
-    TickStatsSnapshot,
     TickStatus,
 )
-from dagster.core.storage.schedules.base import ScheduleStorage
-from dagster.serdes import (
+from dagster._core.storage.schedules.base import ScheduleStorage
+from dagster._serdes import (
     ConfigurableClass,
     ConfigurableClassData,
-    deserialize_as,
-    serialize_dagster_namedtuple,
+    deserialize_value,
+    serialize_value,
 )
+from typing_extensions import Self
 
-from ..errors import GraphQLStorageError
-from .queries import (
+from dagster_cloud.storage.schedules.queries import (
     ADD_JOB_STATE_MUTATION,
     ALL_STORED_JOB_STATE_QUERY,
     CREATE_JOB_TICK_MUTATION,
     GET_JOB_STATE_QUERY,
     GET_JOB_TICKS_QUERY,
-    GET_JOB_TICK_STATS_QUERY,
-    PURGE_JOB_TICKS_MUTATION,
     UPDATE_JOB_STATE_MUTATION,
     UPDATE_JOB_TICK_MUTATION,
 )
 
+if TYPE_CHECKING:
+    from dagster_cloud.instance import DagsterCloudAgentInstance  # noqa: F401
 
-class GraphQLScheduleStorage(ScheduleStorage, ConfigurableClass):
+
+class GraphQLScheduleStorage(ScheduleStorage["DagsterCloudAgentInstance"], ConfigurableClass):
     def __init__(self, inst_data=None, override_graphql_client=None):
         """Initialize this class directly only for test (using `override_graphql_client`).
-        Use the ConfigurableClass machinery to init from instance yaml."""
+        Use the ConfigurableClass machinery to init from instance yaml.
+        """
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
         self._override_graphql_client = override_graphql_client
 
@@ -46,9 +54,9 @@ class GraphQLScheduleStorage(ScheduleStorage, ConfigurableClass):
     def config_type(cls):
         return {}
 
-    @staticmethod
-    def from_config_value(inst_data: ConfigurableClassData, config_value):
-        return GraphQLScheduleStorage(inst_data=inst_data)
+    @classmethod
+    def from_config_value(cls, inst_data: ConfigurableClassData, config_value: Any) -> Self:
+        return cls(inst_data=inst_data)
 
     @property
     def _graphql_client(self):
@@ -59,35 +67,43 @@ class GraphQLScheduleStorage(ScheduleStorage, ConfigurableClass):
         )
 
     def _execute_query(self, query, variables=None):
-        res = self._graphql_client.execute(query, variable_values=variables)
-        if "errors" in res:
-            raise GraphQLStorageError(res)
-        return res
+        return self._graphql_client.execute(query, variable_values=variables)
 
     def wipe(self):
         raise Exception("Not allowed to wipe from user cloud")
 
-    def all_instigator_state(
-        self, repository_origin_id: str = None, instigator_type: InstigatorType = None
+    def all_instigator_state(  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
+        self,
+        repository_origin_id: Optional[str] = None,
+        repository_selector_id: Optional[str] = None,
+        instigator_type: Optional[InstigatorType] = None,
+        instigator_statuses: Optional[set[InstigatorStatus]] = None,
     ) -> Iterable[InstigatorState]:
         res = self._execute_query(
             ALL_STORED_JOB_STATE_QUERY,
             variables={
                 "repositoryOriginId": repository_origin_id,
+                "repositorySelectorId": repository_selector_id,
                 "jobType": instigator_type.value if instigator_type else None,
+                "statuses": (
+                    [status.value for status in instigator_statuses]
+                    if instigator_statuses
+                    else None
+                ),
             },
         )
 
         return [
-            deserialize_as(state, InstigatorState)
+            deserialize_value(state, InstigatorState)
             for state in res["data"]["schedules"]["jobStates"]
         ]
 
-    def get_instigator_state(self, origin_id: str) -> InstigatorState:
+    def get_instigator_state(self, origin_id: str, selector_id: str) -> Optional[InstigatorState]:
         res = self._execute_query(
             GET_JOB_STATE_QUERY,
             variables={
                 "jobOriginId": origin_id,
+                "selectorId": selector_id,
             },
         )
 
@@ -95,60 +111,65 @@ class GraphQLScheduleStorage(ScheduleStorage, ConfigurableClass):
         if state is None:
             return None
 
-        return deserialize_as(state, InstigatorState)
+        return deserialize_value(state, InstigatorState)
 
-    def add_instigator_state(self, state: InstigatorState):
+    def add_instigator_state(self, state: InstigatorState):  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
         self._execute_query(
             ADD_JOB_STATE_MUTATION,
             variables={
-                "serializedJobState": serialize_dagster_namedtuple(
+                "serializedJobState": serialize_value(
                     check.inst_param(state, "state", InstigatorState)
                 )
             },
         )
 
-    def update_instigator_state(self, state: InstigatorState):
+    def update_instigator_state(self, state: InstigatorState):  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
         self._execute_query(
             UPDATE_JOB_STATE_MUTATION,
             variables={
-                "serializedJobState": serialize_dagster_namedtuple(
+                "serializedJobState": serialize_value(
                     check.inst_param(state, "state", InstigatorState)
                 )
             },
         )
 
-    def delete_instigator_state(self, origin_id: str):
+    def delete_instigator_state(self, origin_id: str, selector_id: str):
         raise NotImplementedError("Not callable from user cloud")
 
-    def get_ticks(
+    def get_tick(self, tick_id: int) -> InstigatorTick:
+        raise NotImplementedError("Not callable from user cloud")
+
+    def get_ticks(  # pyright: ignore[reportIncompatibleMethodOverride], fix me!
         self,
         origin_id: str,
-        before: float = None,
-        after: float = None,
-        limit: int = None,
-        statuses: Optional[List[TickStatus]] = None,
+        selector_id: str,
+        before: Optional[float] = None,
+        after: Optional[float] = None,
+        limit: Optional[int] = None,
+        statuses: Optional[Sequence[TickStatus]] = None,
     ) -> Iterable[InstigatorTick]:
-        statuses = [status.value for status in statuses] if statuses else None
+        status_strs = [status.value for status in statuses] if statuses else None
         res = self._execute_query(
             GET_JOB_TICKS_QUERY,
             variables={
                 "jobOriginId": origin_id,
+                "selectorId": selector_id,
                 "before": before,
                 "after": after,
                 "limit": limit,
-                "statuses": statuses,
+                "statuses": status_strs,
             },
         )
 
         return [
-            deserialize_as(tick, InstigatorTick) for tick in res["data"]["schedules"]["jobTicks"]
+            deserialize_value(tick, InstigatorTick) for tick in res["data"]["schedules"]["jobTicks"]
         ]
 
     def create_tick(self, tick_data: TickData):
         res = self._execute_query(
             CREATE_JOB_TICK_MUTATION,
             variables={
-                "serializedJobTickData": serialize_dagster_namedtuple(
+                "serializedJobTickData": serialize_value(
                     check.inst_param(tick_data, "tick_data", TickData)
                 )
             },
@@ -162,41 +183,41 @@ class GraphQLScheduleStorage(ScheduleStorage, ConfigurableClass):
             UPDATE_JOB_TICK_MUTATION,
             variables={
                 "tickId": tick.tick_id,
-                "serializedJobTickData": serialize_dagster_namedtuple(
+                "serializedJobTickData": serialize_value(
                     check.inst_param(tick.tick_data, "tick_data", TickData)
                 ),
             },
         )
         return tick
 
-    def purge_ticks(self, origin_id: str, tick_status: TickStatus, before: float):
-        self._execute_query(
-            PURGE_JOB_TICKS_MUTATION,
-            variables={
-                "jobOriginId": origin_id,
-                "tickStatus": tick_status.value if tick_status else None,
-                "before": before,
-            },
-        )
+    def purge_ticks(
+        self,
+        origin_id: str,
+        selector_id: str,
+        before: float,
+        tick_statuses: Optional[Sequence[TickStatus]] = None,
+    ):
+        raise NotImplementedError("Not callable from user cloud")
 
-    def get_tick_stats(self, origin_id: str):
-        res = self._execute_query(
-            GET_JOB_TICK_STATS_QUERY,
-            variables={
-                "jobOriginId": origin_id,
-            },
-        )
+    def add_auto_materialize_asset_evaluations(
+        self,
+        evaluation_id: int,
+        asset_evaluations: Sequence[AutomationConditionEvaluationWithRunIds],
+    ) -> None:
+        raise NotImplementedError("Not callable from user cloud")
 
-        stats = res["data"]["schedules"]["jobTickStats"]
-        return TickStatsSnapshot(
-            ticks_started=stats["ticksStarted"],
-            ticks_succeeded=stats["ticksSucceeded"],
-            ticks_skipped=stats["ticksSkipped"],
-            ticks_failed=stats["ticksFailed"],
-        )
+    def get_auto_materialize_asset_evaluations(
+        self, key: EntityKey, limit: int, cursor: Optional[int] = None
+    ) -> Sequence[AutoMaterializeAssetEvaluationRecord]:
+        raise NotImplementedError("Not callable from user cloud")
+
+    def get_auto_materialize_evaluations_for_evaluation_id(
+        self, evaluation_id: int
+    ) -> Sequence[AutoMaterializeAssetEvaluationRecord]:
+        raise NotImplementedError("Not callable from user cloud")
+
+    def purge_asset_evaluations(self, before: float):
+        raise NotImplementedError("Not callable from user cloud")
 
     def upgrade(self):
         raise NotImplementedError("Not callable from user cloud")
-
-    def optimize_for_dagit(self, statement_timeout: int):
-        raise NotImplementedError("Not callable from user cloud")
